@@ -10,6 +10,9 @@ open Atd.Ast
 open Mapping
 open Ob_mapping
 
+let target : Ocaml.target = Biniou
+let annot_schema = Ocaml.annot_schema_of_target target
+
 (*
   OCaml code generator (biniou readers and writers)
 *)
@@ -53,21 +56,21 @@ val %s_tag : Bi_io.node_tag
     bprintf buf "\
 val write_untagged_%s :%s
   Bi_outbuf.t -> %s -> unit
-  (** Output an untagged biniou value of type {!%s}. *)
+  (** Output an untagged biniou value of type {!type:%s}. *)
 
 " x.def_name writer_params full_name x.def_name;
 
     bprintf buf "\
 val write_%s :%s
   Bi_outbuf.t -> %s -> unit
-  (** Output a biniou value of type {!%s}. *)
+  (** Output a biniou value of type {!type:%s}. *)
 
 " x.def_name writer_params full_name x.def_name;
 
     bprintf buf "\
 val string_of_%s :%s
   ?len:int -> %s -> string
-  (** Serialize a value of type {!%s} into
+  (** Serialize a value of type {!type:%s} into
       a biniou string. *)
 
 " x.def_name writer_params full_name x.def_name;
@@ -78,21 +81,21 @@ val string_of_%s :%s
 val get_%s_reader :%s
   Bi_io.node_tag -> (Bi_inbuf.t -> %s)
   (** Return a function that reads an untagged
-      biniou value of type {!%s}. *)
+      biniou value of type {!type:%s}. *)
 
 " x.def_name reader_params full_name x.def_name;
 
     bprintf buf "\
 val read_%s :%s
   Bi_inbuf.t -> %s
-  (** Input a tagged biniou value of type {!%s}. *)
+  (** Input a tagged biniou value of type {!type:%s}. *)
 
 " x.def_name reader_params full_name x.def_name;
 
     bprintf buf "\
 val %s_of_string :%s
   ?pos:int -> string -> %s
-  (** Deserialize a biniou value of type {!%s}.
+  (** Deserialize a biniou value of type {!type:%s}.
       @param pos specifies the position where
                  reading starts. Default: 0. *)
 
@@ -1359,11 +1362,13 @@ let make_ocaml_files
     match atd_file with
       Some file ->
         Atd.Util.load_file
+          ~annot_schema
           ~expand:false ~inherit_fields:true ~inherit_variants:true
           ?pos_fname ?pos_lnum
           file
     | None ->
         Atd.Util.read_channel
+          ~annot_schema
           ~expand:false ~inherit_fields:true ~inherit_variants:true
           ?pos_fname ?pos_lnum
           stdin
@@ -1385,7 +1390,7 @@ let make_ocaml_files
      m1 = original type definitions after dependency analysis
      m2 = monomorphic type definitions after dependency analysis *)
   let ocaml_typedefs =
-    Ocaml.ocaml_of_atd ~pp_convs ~target:Biniou ~type_aliases (head, m1) in
+    Ocaml.ocaml_of_atd ~pp_convs ~target ~type_aliases (head, m1) in
   let defs = defs_of_atd_modules m2 in
   let header =
     let src =
@@ -1394,7 +1399,7 @@ let make_ocaml_files
       | Some path -> sprintf "%S" (Filename.basename path)
     in
     sprintf {|(* Auto-generated from %s *)
-              [@@@ocaml.warning "-27-32-35-39"]|} src
+[@@@ocaml.warning "-27-32-33-35-39"]|} src
   in
   let mli =
     make_mli ~header ~opens ~with_typedefs ~with_create ~with_fundefs
