@@ -2,6 +2,7 @@
 
 type pp_convs =
   | Camlp4 of string list
+  | Ppx_deriving of string list
   | Ppx of string list
 
 type atd_ocaml_sum = Classic | Poly
@@ -41,6 +42,7 @@ module Repr : sig
     | Int of atd_ocaml_int
     | Float
     | String
+    | Abstract
     | Sum of atd_ocaml_sum
     | Record of atd_ocaml_record
     | Tuple
@@ -62,6 +64,8 @@ module Repr : sig
     | Def of atd_ocaml_def
 end
 
+val annot_schema_of_target : target -> Atd.Annot.schema
+
 val get_ocaml_sum : target -> Atd.Annot.t -> atd_ocaml_sum
 val get_ocaml_record : target -> Atd.Annot.t -> atd_ocaml_record
 val get_ocaml_field_prefix : target -> Atd.Annot.t -> string
