@@ -43,7 +43,7 @@ let make_ocaml_json_intf ~with_create buf deref defs =
     let writer_params =
       String.concat "" (
         List.map
-          (fun s -> sprintf "\n  (Bi_outbuf.t -> '%s -> unit) ->" s)
+          (fun s -> sprintf "\n  (Buffer.t -> '%s -> unit) ->" s)
           x.def_param
       )
     in
@@ -58,7 +58,7 @@ let make_ocaml_json_intf ~with_create buf deref defs =
     in
     bprintf buf "\
 val write_%s :%s
-  Bi_outbuf.t -> %s -> unit
+  Buffer.t -> %s -> unit
   (** Output a JSON value of type {!type:%s}. *)
 
 "
@@ -322,7 +322,7 @@ let rec make_writer ?type_constraint p (x : Oj_mapping.t) : Indent.t list =
         ) a
       in
       let l =
-        insert (Line "Bi_outbuf.add_char ob ',';") (Array.to_list a)
+        insert (Line "Buffer.add_char ob ',';") (Array.to_list a)
       in
       let op, cl =
         if p.std then '[', ']'
@@ -331,9 +331,9 @@ let rec make_writer ?type_constraint p (x : Oj_mapping.t) : Indent.t list =
       [
         Annot ("fun", Line "fun ob x ->");
         Block [
-          Line (sprintf "Bi_outbuf.add_char ob %C;" op);
+          Line (sprintf "Buffer.add_char ob %C;" op);
           Inline l;
-          Line (sprintf "Bi_outbuf.add_char ob %C;" cl);
+          Line (sprintf "Buffer.add_char ob %C;" cl);
         ]
       ]
 
@@ -415,7 +415,7 @@ and make_variant_writer p ~tick ~open_enum x : Indent.t list =
         else "<" ^ s ^ ">"
       in
       [
-        Line (sprintf "| %s%s -> Bi_outbuf.add_string ob %S"
+        Line (sprintf "| %s%s -> Buffer.add_string ob %S"
                 tick ocaml_cons
                 (enclose (make_json_string json_cons)))
       ]
@@ -436,12 +436,12 @@ and make_variant_writer p ~tick ~open_enum x : Indent.t list =
       [
         Line (sprintf "| %s%s x ->" tick ocaml_cons);
         Block [
-          Line (sprintf "Bi_outbuf.add_string ob %S;"
+          Line (sprintf "Buffer.add_string ob %S;"
                   (op ^ make_json_string json_cons ^ sep));
           Line "(";
           Block (make_writer p v);
           Line ") ob x;";
-          Line (sprintf "Bi_outbuf.add_char ob %C" cl);
+          Line (sprintf "Buffer.add_char ob %C" cl);
         ]
       ]
 
@@ -456,7 +456,7 @@ and make_record_writer p a record_kind =
       ];
       Line "else";
       Block [
-        Line "Bi_outbuf.add_char ob ',';";
+        Line "Buffer.add_char ob ',';";
       ];
     ]
   in
@@ -467,7 +467,7 @@ and make_record_writer p a record_kind =
            ; json_fname ; optional ; unwrapped } as field) ->
         let f_value = unwrap_f_value field p in
         let write_field_tag =
-          sprintf "Bi_outbuf.add_string ob %S;"
+          sprintf "  Buffer.add_string ob %S;"
             (make_json_string json_fname ^ ":")
         in
         let app v =
@@ -507,10 +507,10 @@ and make_record_writer p a record_kind =
     ) fields
   in
   [
-    Line "Bi_outbuf.add_char ob '{';";
+    Line "Buffer.add_char ob '{';";
     Line "let is_first = ref true in";
     Inline write_fields;
-    Line "Bi_outbuf.add_char ob '}';";
+    Line "Buffer.add_char ob '}';";
   ]
 
 let wrap_tmp_required_value value =
@@ -1200,9 +1200,9 @@ let make_ocaml_json_writer p ~original_types is_rec let1 let2 def =
     Line (sprintf ")%s" extra_args);
     Line (sprintf "%s %s ?(len = 1024) x =" let2 to_string);
     Block [
-      Line "let ob = Bi_outbuf.create len in";
+      Line "let ob = Buffer.create len in";
       Line (sprintf "%s ob x;" write);
-      Line "Bi_outbuf.contents ob"
+      Line "Buffer.contents ob"
     ]
   ]
 
