@@ -3,6 +3,8 @@ open Atds_names
 open Atds_env
 open Atds_util
 
+module A = Atd.Ast
+
 (* Declare a case class field, with support for optional fields.  The are two
  * kinds of optional fields: `With_default (~) and `Optional (?).  For both
  * kinds, we return the following values if the field is absent:
@@ -34,7 +36,7 @@ let declare_field env
   let field_name = get_scala_field_name atd_field_name annots in
   let opt_default =
     match kind with
-    | Atd.Ast.With_default ->
+    | A.With_default ->
         (match norm_ty env atd_ty with
          | Name (_, (_, name, _), _) ->
              (match name with
@@ -132,7 +134,7 @@ package object %s {
 
 let rec trans_module env items = List.fold_left trans_outer env items
 
-and trans_outer env (Atd.Ast.Type (_, (name, _, annots), atd_ty)) =
+and trans_outer env (A.Type (_, (name, _, annots), atd_ty)) =
   match unwrap atd_ty with
   | Sum (loc, v, a) ->
       trans_sum name env (loc, v, a)
@@ -152,18 +154,20 @@ and trans_outer env (Atd.Ast.Type (_, (name, _, annots), atd_ty)) =
 and trans_sum my_name env (_, vars, _) =
   let class_name = Atds_names.to_class_name my_name in
 
-  let cases = List.map (function
-    | Atd.Ast.Variant (_, (atd_name, an), opt_ty) ->
-        let json_name = get_json_variant_name atd_name an in
-        let scala_name = get_scala_variant_name atd_name an in
-        let opt_java_ty =
-          opt_ty |> Option.map (fun ty ->
-            let java_ty = trans_inner env ty in
-            (ty, java_ty)
-          ) in
-        (json_name, scala_name, opt_java_ty)
-    | Inherit _ -> assert false
-  ) vars
+  let cases =
+    List.map (fun (x : A.variant) ->
+      match x with
+      | Variant (_, (atd_name, an), opt_ty) ->
+          let json_name = get_json_variant_name atd_name an in
+          let scala_name = get_scala_variant_name atd_name an in
+          let opt_java_ty =
+            opt_ty |> Option.map (fun ty ->
+              let java_ty = trans_inner env ty in
+              (ty, java_ty)
+            ) in
+          (json_name, scala_name, opt_java_ty)
+      | Inherit _ -> assert false
+    ) vars
   in
 
   let out = env.output in
